@@ -1,70 +1,48 @@
 use crate::{
   activities::{
-    community::announce::AnnouncableActivities,
-    deletion::{
-      delete::Delete,
-      receive_delete_action,
-      verify_delete_activity,
-      DeletableObjects,
-      WebsocketMessages,
-    },
+    community::{announce::GetCommunity, send_activity_in_community},
+    deletion::{receive_delete_action, verify_delete_activity, DeletableObjects},
     generate_activity_id,
     verify_activity,
+    verify_is_public,
   },
-  activity_queue::send_to_community_new,
-  extensions::context::lemmy_context,
-  fetcher::object_id::ObjectId,
-  ActorType,
-};
-use activitystreams::{
-  activity::kind::UndoType,
-  base::AnyBase,
-  primitives::OneOrMany,
-  unparsed::Unparsed,
+  activity_lists::AnnouncableActivities,
+  objects::{community::ApubCommunity, person::ApubPerson},
+  protocol::activities::deletion::{delete::Delete, undo_delete::UndoDelete},
 };
+use activitystreams::{activity::kind::UndoType, public};
 use anyhow::anyhow;
 use lemmy_api_common::blocking;
-use lemmy_apub_lib::{values::PublicUrl, ActivityFields, ActivityHandler};
-use lemmy_db_queries::source::{comment::Comment_, community::Community_, post::Post_};
-use lemmy_db_schema::source::{comment::Comment, community::Community, person::Person, post::Post};
+use lemmy_apub_lib::{
+  data::Data,
+  object_id::ObjectId,
+  traits::{ActivityHandler, ActorType},
+};
+use lemmy_db_schema::source::{comment::Comment, community::Community, post::Post};
 use lemmy_utils::LemmyError;
 use lemmy_websocket::{
   send::{send_comment_ws_message_simple, send_community_ws_message, send_post_ws_message},
   LemmyContext,
   UserOperationCrud,
 };
-use serde::{Deserialize, Serialize};
 use url::Url;
 
-#[derive(Clone, Debug, Deserialize, Serialize, ActivityFields)]
-#[serde(rename_all = "camelCase")]
-pub struct UndoDelete {
-  actor: ObjectId<Person>,
-  to: [PublicUrl; 1],
-  object: Delete,
-  cc: [ObjectId<Community>; 1],
-  #[serde(rename = "type")]
-  kind: UndoType,
-  id: Url,
-  #[serde(rename = "@context")]
-  context: OneOrMany<AnyBase>,
-  #[serde(flatten)]
-  unparsed: Unparsed,
-}
-
 #[async_trait::async_trait(?Send)]
 impl ActivityHandler for UndoDelete {
+  type DataType = LemmyContext;
   async fn verify(
     &self,
-    context: &LemmyContext,
+    context: &Data<LemmyContext>,
     request_counter: &mut i32,
   ) -> Result<(), LemmyError> {
-    verify_activity(self, &context.settings())?;
+    verify_is_public(&self.to, &self.cc)?;
+    verify_activity(&self.id, self.actor.inner(), &context.settings())?;
     self.object.verify(context, request_counter).await?;
+    let community = self.get_community(context, request_counter).await?;
     verify_delete_activity(
-      &self.object.object,
-      self,
-      &self.cc[0],
+      &self.object.object.id,
+      &self.actor,
+      &community,
       self.object.summary.is_some(),
       context,
       request_counter,
@@ -75,20 +53,15 @@ impl ActivityHandler for UndoDelete {
 
   async fn receive(
     self,
-    context: &LemmyContext,
+    context: &Data<LemmyContext>,
     request_counter: &mut i32,
   ) -> Result<(), LemmyError> {
     if self.object.summary.is_some() {
-      UndoDelete::receive_undo_remove_action(&self.object.object, context).await
+      UndoDelete::receive_undo_remove_action(&self.object.object.id, context).await
     } else {
       receive_delete_action(
-        &self.object.object,
+        &self.object.object.id,
         &self.actor,
-        WebsocketMessages {
-          community: UserOperationCrud::EditCommunity,
-          post: UserOperationCrud::EditPost,
-          comment: UserOperationCrud::EditComment,
-        },
         false,
         context,
         request_counter,
@@ -100,13 +73,13 @@ impl ActivityHandler for UndoDelete {
 
 impl UndoDelete {
   pub(in crate::activities::deletion) async fn send(
-    actor: &Person,
-    community: &Community,
-    object_id: Url,
+    actor: &ApubPerson,
+    community: &ApubCommunity,
+    object: DeletableObjects,
     summary: Option<String>,
     context: &LemmyContext,
   ) -> Result<(), LemmyError> {
-    let object = Delete::new(actor, community, object_id, summary, context)?;
+    let object = Delete::new(actor, object, summary, context)?;
 
     let id = generate_activity_id(
       UndoType::Undo,
@@ -114,17 +87,16 @@ impl UndoDelete {
     )?;
     let undo = UndoDelete {
       actor: ObjectId::new(actor.actor_id()),
-      to: [PublicUrl::Public],
+      to: vec![public()],
       object,
-      cc: [ObjectId::new(community.actor_id())],
+      cc: vec![community.actor_id()],
       kind: UndoType::Undo,
       id: id.clone(),
-      context: lemmy_context(),
       unparsed: Default::default(),
     };
 
     let activity = AnnouncableActivities::UndoDelete(undo);
-    send_to_community_new(activity, &id, actor, community, vec![], context).await
+    send_activity_in_community(activity, &id, actor, community, vec![], context).await
   }
 
   pub(in crate::activities) async fn receive_undo_remove_action(
@@ -161,3 +133,14 @@ impl UndoDelete {
     Ok(())
   }
 }
+
+#[async_trait::async_trait(?Send)]
+impl GetCommunity for UndoDelete {
+  async fn get_community(
+    &self,
+    context: &LemmyContext,
+    request_counter: &mut i32,
+  ) -> Result<ApubCommunity, LemmyError> {
+    self.object.get_community(context, request_counter).await
+  }
+}
